/*
** Copyright (C) 1995, 1996, 1997, 1998 Hewlett-Packard Company
** Originally by Kevin Hughes, kev@kevcom.com, 3/11/94
**
** This program and library is free software; you can redistribute it and/or
** modify it under the terms of the GNU (Library) General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU (Library) General Public License for more details.
**
** You should have received a copy of the GNU (Library) General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
**-------------------------------------------------------------
** Changed getdefaults to allow metaNames in the user
** configuration file
** G.Hill 4/16/97 ghill@library.berkeley.edu
*/

#include "swish.h"
#include "file.h"
#include "mem.h"
#include "string.h"
#include "error.h"
#include "list.h"
#include "hash.h"
#include "index.h"

/* Is a file a directory?
*/

int isdirectory(path)
char *path;
{
	struct stat stbuf;
	
	if (stat(path, &stbuf))
		return 0;
	return ((stbuf.st_mode & S_IFMT) == S_IFDIR) ? 1 : 0;
}

/* Is a file a regular file?
*/

int isfile(path)
char *path;
{
	struct stat stbuf;
	
	if (stat(path, &stbuf))
		return 0;
	return ((stbuf.st_mode & S_IFMT) == S_IFREG) ? 1 : 0;
}

/* Is a file a link?
*/

int islink(path)
char *path;
{
#ifndef NO_SYMBOLIC_FILE_LINKS
	struct stat stbuf;
	
	if (lstat(path, &stbuf))
		return 0;
	return ((stbuf.st_mode & S_IFLNK) == S_IFLNK) ? 1 : 0;
#else
	return 0;
#endif
}

/* Get the size, in bytes, of a file.
** Return -1 if there's a problem.
*/

int getsize(path)
char *path;
{
	struct stat stbuf;
	
	if (stat(path, &stbuf))
		return -1;
	return stbuf.st_size;
}

/* Add an entry to the metaEntryList with the given value and the
** appropriate index
*/

struct metaEntry* addMetaEntry(metaList, metaWord)
struct metaEntry* metaList;
char* metaWord;
{
	static int counter;
	int i;
	struct metaEntry* newEntry;
	struct metaEntry* tmpEntry;
	
	if (counter == 0)
		counter = 2;
	else if (counter == 1 ||  (!counter % 128) )
		counter++;
	for( i=0; metaWord[i]; i++)
		metaWord[i] =  tolower(metaWord[i]);
	newEntry = (struct metaEntry*) emalloc(sizeof(struct metaEntry));
	newEntry->metaName = (char*)mystrdup(metaWord);
	newEntry->index = counter++;
	newEntry->next = NULL;
	if (metaEntryList)
    {
		for(tmpEntry=metaEntryList;tmpEntry->next!=NULL;tmpEntry=tmpEntry->next)
			;
		tmpEntry->next = newEntry;
    }
	else
		metaEntryList = newEntry;
	
	return metaEntryList;
}

/* Reads the configuration file and puts all the right options
** in the right variables and structures.
*/

void getdefaults(conffile, hasdir, hasindex, plimit, flimit, hasverbose)
char *conffile;
int *hasdir;
int *hasindex;
long *plimit;
long *flimit;
int hasverbose;
{
	int skiplen, gotdir, gotindex;
	char *c, line[MAXSTRLEN], value[MAXSTRLEN];
	FILE *fp;
	int linenumber = 0;
	int baddirective = 0;
	
	gotdir = gotindex = 0;
	
	if ((fp = fopen(conffile, "r")) == NULL  ||
		!isfile(conffile) ) 
	{
		sprintf(errorstr, "Couldn't open the configuration file \"%s\".", conffile);
		progerr(errorstr);
	}

	while (fgets(line, MAXSTRLEN, fp) != NULL) 
	{
		linenumber++;
		if (line[0] == '#' || line[0] == '\n')
			continue;
		if (c = (char *) lstrstr(line, "IndexDir")) {
			c += strlen("IndexDir");
			while (!*hasdir) {
				strcpy(value, (char *) getword(c, &skiplen));
				if (!skiplen || value[0] == '\0' ||
					value[0] == '\n')
					break;
				else {
					c += skiplen;
					dirlist = (struct swline *)
						addswline(dirlist, value);
					gotdir = 1;
				}
			}
		}
		else if (c = (char *) lstrstr(line, "IndexFile")) {
			c += strlen("IndexFile");
			while (!*hasindex) {
				strcpy(value, (char *) getword(c, &skiplen));
				if (!skiplen || value[0] == '\0' ||
					value[0] == '\n')
					break;
				else {
					c += skiplen;
					indexlist = (struct swline *)
						addswline(indexlist, value);
					gotindex = 1;
				}
			}
		}
		/* IndexVerbose is supported for backwards compatibility */
		else if (c = (char *) lstrstr(line, "IndexVerbose")) {
			c += strlen("IndexVerbose");
			strcpy(value, (char *) getword(c, &skiplen));
			verbose = (lstrstr(value, "yes")) ? 3 : 0;
		}
		else if (c = (char *) lstrstr(line, "IndexReport")) {
			c += strlen("IndexReport");
			strcpy(value, (char *) getword(c, &skiplen));
			if (!skiplen || value[0] == '\0' || value[0] == '\n')
				continue;
			else {
				c += skiplen;
				if (!hasverbose)
					verbose = atoi(value);
			}
		}
		else if (c = (char *) lstrstr(line, "MinWordLimit")) {
			c += strlen("MinWordLimit");
			strcpy(value, (char *) getword(c, &skiplen));
			if (!skiplen || value[0] == '\0' || value[0] == '\n')
				continue;
			else {
				c += skiplen;
				minwordlimit = atoi(value);
			}
		}
		else if (c = (char *) lstrstr(line, "IndexComments")) {
			c += strlen("IndexComments");
			strcpy(value, (char *) getword(c, &skiplen));
			if (!skiplen || value[0] == '\0' || value[0] == '\n')
				continue;
			else {
				c += skiplen;
				indexComments = atoi(value);
			}
		}
		else if (c = (char *) lstrstr(line, "MaxWordLimit")) {
			c += strlen("MaxWordLimit");
			strcpy(value, (char *) getword(c, &skiplen));
			if (!skiplen || value[0] == '\0' || value[0] == '\n')
				continue;
			else {
				c += skiplen;
				maxwordlimit = atoi(value);
			}
		}
		else if (c = (char *) lstrstr(line, "WordCharacters")) {
			c += strlen("WordCharacters");
			strcpy(value, (char *) getword(c, &skiplen));
			if (!skiplen || value[0] == '\0' || value[0] == '\n')
				continue;
			else {
				c += skiplen;
				strcpy(wordchars,value);
			}
		}
		else if (c = (char *) lstrstr(line, "BeginCharacters")) {
			c += strlen("BeginCharacters");
			strcpy(value, (char *) getword(c, &skiplen));
			if (!skiplen || value[0] == '\0' || value[0] == '\n')
				continue;
			else {
				c += skiplen;
				strcpy(beginchars,value);
			}
		}
		else if (c = (char *) lstrstr(line, "EndCharacters")) {
			c += strlen("EndCharacters");
			strcpy(value, (char *) getword(c, &skiplen));
			if (!skiplen || value[0] == '\0' || value[0] == '\n')
				continue;
			else {
				c += skiplen;
				strcpy(endchars,value);
			}
		}
		else if (c = (char *) lstrstr(line, "IgnoreLastChar")) {
			c += strlen("IgnoreLastChar");
			strcpy(value, (char *) getword(c, &skiplen));
			if (!skiplen || value[0] == '\0' || value[0] == '\n'){
				ignorelastchar[0] = '\0';
				continue;
			}
			else {
				c += skiplen;
				strcpy(ignorelastchar,value);
			}
		}
		else if (c = (char *) lstrstr(line, "IgnoreFirstChar")) {
			c += strlen("IgnoreFirstChar");
			strcpy(value, (char *) getword(c, &skiplen));
			if (!skiplen || value[0] == '\0' || value[0] == '\n') {
				ignorefirstchar[0] = '\0';
				continue;
			}
			else {
				c += skiplen;
				strcpy(ignorefirstchar,value);
			}
		}
		else if (c = (char *) lstrstr(line, "MetaNames")) {
			c += strlen("MetaNames");
			while (1) {
				strcpy(value, (char *) getword(c, &skiplen));
				if (!skiplen || value[0] == '\0' ||
					value[0] == '\n')
					break;
				else {
					c += skiplen;
					metaEntryList = (struct metaEntry *)
						addMetaEntry(metaEntryList, value);
				}
			}
		      }
		else if (c = (char *) lstrstr(line, "IgnoreWords")) {
			c += strlen("IgnoreWords");
			while (1) {
				strcpy(value, (char *) getword(c, &skiplen));
				if (!skiplen || value[0] == '\0' ||
					value[0] == '\n')
					break;
				else {
					c += skiplen;
					if (lstrstr(value, "SwishDefault"))
						readdefaultstopwords();
					else
						addstophash(value);
				}
			}
		}
		else if ((c = (char *) lstrstr(line, "IgnoreLimit"))) {
			c += strlen("IgnoreLimit");
			strcpy(value, (char *) getword(c, &skiplen));
			if (!skiplen || value[0] == '\0' || value[0] == '\n')
				continue;
			else {
				c += skiplen;
				*plimit = atoi(value);
			}
			strcpy(value, (char *) getword(c, &skiplen));
			if (!skiplen || value[0] == '\0' || value[0] == '\n')
				continue;
			else {
				c += skiplen;
				*flimit = atoi(value);
			}
		}
		else if (c = (char *) lstrstr(line, "ReplaceRules")) {
			c += strlen("ReplaceRules");
			while (1) {
				strcpy(value, (char *) getword(c, &skiplen));
				if (!skiplen || value[0] == '\0' ||
					value[0] == '\n')
					break;
				else {
					c += skiplen;
					replacelist = (struct swline *)
						addswline(replacelist, value);
				}
			}
			checkReplaceList();
		}
		else if ((char *) getconfvalue(line, "FollowSymLinks",
					       value) != NULL)
						   followsymlinks = (lstrstr(value, "yes")) ? 1 : 0;
		else if ((char *) getconfvalue(line, "IndexName",
					       value) != NULL)
						   strcpy(indexn, value);
		else if ((char *) getconfvalue(line, "IndexDescription",
					       value) != NULL)
						   strcpy(indexd, value);
		else if ((char *) getconfvalue(line, "IndexPointer",
					       value) != NULL)
						   strcpy(indexp, value);
		else if ((char *) getconfvalue(line, "IndexAdmin",
					       value) != NULL)
						   strcpy(indexa, value);
		else if (!parseconfline(line)) {
			printf("Bad directive on line #%d: %s", linenumber, line );
			baddirective = 1;
		}
	}
	fclose(fp);
	
	if (baddirective)
		exit(1);
	if (gotdir && !(*hasdir))
		*hasdir = 1;
	if (gotindex && !(*hasindex))
		*hasindex = 1;
}

/* Checks that all the regex in the replace list are correct */
void checkReplaceList() 
{
	struct swline *tmpReplace;
	char rule[MAXSTRLEN], patt[MAXSTRLEN];
	regex_t re;
	int status;
	
	tmpReplace = replacelist;
	while (tmpReplace) {
		strcpy(rule,tmpReplace->line);
		
		/*  If it is not replace, just do nothing */
		if (lstrstr(rule,"append") || lstrstr(rule,"prepend") ) {
			if (tmpReplace->next){
				tmpReplace = tmpReplace->next;
			}
			else
				return;
		}
		if (lstrstr(rule,"replace")) {
			tmpReplace = tmpReplace->next;
			strcpy(patt,tmpReplace->line);
			if (patt == NULL) 
				return;
			status = regcomp(&re,patt, REG_EXTENDED);
			if (status != 0) {
				printf ("Illegal regular expression %s\n", patt);
				exit(0);
			}
			
			if (tmpReplace->next) 
				tmpReplace = tmpReplace->next;
			else
				return;
		}
		tmpReplace = tmpReplace->next;
	}
}

/* This is similar to the previous one, just kept separated because */
/* of the different structure of the list                           */
void checkListRegex (list) 
struct swline *list;
{
	struct swline *tmpReplace;
	char patt[MAXSTRLEN];
	regex_t re;
	int status;
	
	tmpReplace = replacelist;
	while (tmpReplace) {
		strcpy(patt,tmpReplace->line);
		if (patt == NULL) 
			return;
		status = regcomp(&re,patt, REG_EXTENDED);
		if (status != 0) {
			printf ("Illegal regular expression %s\n", patt);
			exit(0);
		}
		tmpReplace = tmpReplace->next;
	}
}/* end of checkListRegex */




